// parameter class definitions
// automatically generated by make-parameter-class
// part of libecsb - the executable cell systems biology library
// copyright 2008 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#include "parameters.h"

#include <string.h>
#include <stdlib.h>

using namespace ECSB;
using namespace PMODELCHECKER;

// constructor
MCParameters::MCParameters(void)
{
    // set initial values
    this->model_checker = "PRISM";
    this->PRISM_model = "PRISM_model.sm";
    this->simulations_generatedHDF5 = "false";
    this->simulations_generatedMC2 = "false";
    this->molecular_species = "all";
    this->positions = "all";
    this->parameters_file = "modelParameters.xml";
    this->states_file = "states.psm";
    this->transitions_file = "transitions.psm";
    this->task = "Translate";
    this->results_file = "results.psm";
}

//read parameters from a file
void MCParameters::read(std::string xmlFile)
{
    //Use of the DOM parser
    DomParser parser;

    //Parse our XML file.
    parser.parse_file(xmlFile);

    if(!parser)
    {
      cerr << "Parsing Unsuccesfull" << std::endl;
      exit(EXIT_FAILURE);
    }

    //Get the root node of the parameters
    xmlpp::Node* parametersDeclaration = parser.get_document()->get_root_node();

    //Get the declaration of the different parameters
    xmlpp::Node::NodeList component = parametersDeclaration->get_children("parameterSet");
    xmlpp::Node* parameterSet = component.front();
    xmlpp::Node::NodeList parameters = parameterSet->get_children("parameter");

    for(xmlpp::Node::NodeList::iterator par = parameters.begin(); par != parameters.end(); ++par)
    {
        xmlpp::Element* parElement = dynamic_cast<xmlpp::Element*>(*par);
        std::string parameterName = (parElement->get_attribute("name"))->get_value();

        if(parameterName == "model_specification")
        {
            this->model_specification = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "model_checker")
        {
            this->model_checker = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "PRISM_model")
        {
            this->PRISM_model = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "pathMC2")
        {
            this->pathMC2 = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "simulations_generatedHDF5")
        {
            this->simulations_generatedHDF5 = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "simulations_generatedMC2")
        {
            this->simulations_generatedMC2 = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "simulations_file_hdf5")
        {
            this->simulations_file_hdf5 = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "mcss_params_file")
        {
            this->mcss_params_file = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "molecular_species")
        {
            this->molecular_species = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "positions")
        {
            this->positions = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "simulations_file_MC2")
        {
            this->simulations_file_MC2 = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "parameters_file")
        {
            this->parameters_file = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "confidence")
        {
            this->confidence = fromStringTo<double>((parElement->get_attribute("value"))->get_value());
        }
        else if (parameterName == "formula_parameters")
        {
            this->formula_parameters = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "model_parameters")
        {
            this->model_parameters = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "number_samples")
        {
            this->number_samples = fromStringTo<unsigned long>((parElement->get_attribute("value"))->get_value());
        }
        else if (parameterName == "precision")
        {
            this->precision = fromStringTo<double>((parElement->get_attribute("value"))->get_value());
        }
        else if (parameterName == "results_file")
        {
            this->results_file = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "states_file")
        {
            this->states_file = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "task")
        {
            this->task = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "temporal_formulas")
        {
            this->temporal_formulas = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "transitions_file")
        {
            this->transitions_file = (parElement->get_attribute("value"))->get_value();
        }
        else
        {
            cerr << "Unknown parameter " << parameterName <<std::endl;
            exit(EXIT_FAILURE);
        }

    }
}

// read parameters from an xml file and command line
void MCParameters::read(int argc, char **argv)
{
    int ind;
    char pName[1024], *pVal;

    // read parameters from file
    read(string(argv[1]));

    // read parameters specified on command line
    ind = 2;

    while(ind < argc)
    {
        std::string parameterDeclaration = string(argv[ind]);
        size_t pos = parameterDeclaration.find("=");
        if(pos == parameterDeclaration.npos)
        {
            cerr << "error: couldn't parse command line parameter ";
            cerr << argv[ind] << std::endl;
            cerr << "parameter must be specified as: parameterName=parameterValue" << std::endl;
            exit(EXIT_FAILURE);
        }
        else
        {
/*            std::string doubleDash = parameterDeclaration.substr(0,2);
            if(doubleDash != "--")
            {
                cerr << "error: couldn't parse command line parameter ";
                cerr << argv[ind] << std::endl;
                cerr << "parameter must be specified as: --parameterName=parameterValue" << std::endl;
                exit(EXIT_FAILURE);
            }
            std::string parameterName = parameterDeclaration.substr(2,pos-2);*/
            std::string parameterName = parameterDeclaration.substr(0,pos);
            std::string parameterValue = parameterDeclaration.substr(pos+1);
            if(!setParameter(parameterName,parameterValue))
            {
                cerr << "error: unknown parameter " << parameterName << std::endl;
                exit(EXIT_FAILURE);
            }
        }

        ind++;
    }
}

//set parameter
bool MCParameters::setParameter(std::string parameterName,
                                std::string parameterValue)
{

    if(parameterName == "model_specification")
    {
        this->model_specification = parameterValue;
        return true;
    }
    else if (parameterName == "model_checker")
    {
        this->model_checker = parameterValue;
        return true;
    }
    else if (parameterName == "PRISM_model")
    {
        this->PRISM_model = parameterValue;
        return true;
    }
    else if (parameterName == "pathMC2")
    {
        this->pathMC2 = parameterValue;
        return true;
    }
    else if (parameterName == "simulations_generatedHDF5")
    {
        this->simulations_generatedHDF5 = parameterValue;
        return true;
    }
    else if (parameterName == "simulations_generatedMC2")
    {
        this->simulations_generatedMC2 = parameterValue;
        return true;
    }
    else if (parameterName == "simulations_file_hdf5")
    {
        this->simulations_file_hdf5 = parameterValue;
        return true;
    }
    else if (parameterName == "mcss_params_file")
    {
        this->mcss_params_file = parameterValue;
        return true;
    }
    else if (parameterName == "simulations_file_MC2")
    {
        this->simulations_file_MC2 = parameterValue;
        return true;
    }
    else if (parameterName == "molecular_species")
    {
        this->molecular_species = parameterValue;
        return true;
    }
    else if (parameterName == "positions")
    {
        this->positions = parameterValue;
        return true;
    }
    else if (parameterName == "parameters_file")
    {
        this->parameters_file = parameterValue;
        return true;
    }
    else if (parameterName == "confidence")
    {
        this->confidence = fromStringTo<double>(parameterValue);
        return true;
    }
    else if (parameterName == "formula_parameters")
    {
        this->formula_parameters = parameterValue;
        return true;
    }
    else if (parameterName == "model_parameters")
    {
        this->model_parameters = parameterValue;
        return true;
    }
    else if (parameterName == "number_samples")
    {
        this->number_samples = fromStringTo<unsigned long>(parameterValue);
        return true;
    }
    else if (parameterName == "precision")
    {
        this->precision = fromStringTo<double>(parameterValue);
        return true;
    }
    else if (parameterName == "results_file")
    {
        this->results_file = parameterValue;
        return true;
    }
    else if (parameterName == "states_file")
    {
        this->states_file = parameterValue;
        return true;
    }
    else if (parameterName == "task")
    {
        this->task = parameterValue;
        return true;
    }
    else if (parameterName == "temporal_formulas")
    {
        this->temporal_formulas = parameterValue;
        return true;
    }
    else if (parameterName == "transitions_file")
    {
        this->transitions_file = parameterValue;
        return true;
    }
    else
    {
       cerr << "Unknown parameter papas" << parameterName <<std::endl;
       return false;
    }

}

