/*
 * Date (yymmdd):               090320
 * Version:                     0.2
 * Written by:                  Dr. Francisco J. Romero-Campero
 * Additional contributions by: Jonathan Blakes
 * (in no particular order)     Dr. Hongqing Cao
 *                              Dr. Natalio Krasnogor
 *				Dr. Jamie Twycross
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
 */

#ifndef _PRISMMODEL_H
#define	_PRISMMODEL_H

#include<map>
#include<string>

#include <ecsb/ecsb.h>
#include <pmodelchecker/prismmodule.h>
#include <pmodelchecker/prismcommand.h>

using namespace std;

namespace PMODELCHECKER{

class PrismModel {

public:
    PrismModel();
    PrismModel(const PrismModel& orig);
    PrismModel(LPPsystem Pmodel);
    virtual ~PrismModel();

    void write(std::string fileName);
    void writeParameters(std::string fileName);
    void show();

protected:
    void createPRISMmodules(std::map<std::vector<long>,std::string> positions,
            std::map<std::string,ECSB::PSystem> Psystems);
    void distributeCommands(std::map<std::vector<long>,std::string> positions,
			       std::map<std::string,ECSB::PSystem> Psystems);
    void addConstantDouble(std::string constantName, double value);
    void addConstantInt(std::string constantName);
    std::string targetModuleName(ECSB::LPPRule rule,
        std::vector<long> position,
        std::map<std::vector<long>,std::string> positions);

    bool vectorEqual(std::vector<long> vector1, std::vector<long> vector2);

private:
    std::string type;
    std::map<std::string,double> constantsDouble;
    std::map<std::string,std::string> rulesDescription;
    std::vector<std::string> constantsInt;
    std::map<std::string,PrismModule> modules;

};

}

#endif	/* _PRISMMODEL_H */

